#include "tty.h"
#include "keymap.h"

swMenuItem MENUITEM[]=
{
  {"About",998},
  {"Information",997},
  {"Clear History",101},
  {"Add IME/OME",102},
  {"Remove IME",103},
  {"Remove OME",104},
  {"Run Module",105},
  {"Exit",999}
};

swMenu MENU={ 8 , MENUITEM };

int die(char *err,...)
{
  char buf[1024];
  va_list argptr;
  
  va_start(argptr,err);
  vsprintf(buf,err,argptr);
  va_end(argptr);

  printf("%s\n",buf);
  exit(-1);
}

char * AssignString(iomeConfig *CFG,char *item,char *err)
{
  char * v;
  v = iomeWhichItem(CFG,item);
  if (v==NULL) die(err);
  return v;
}

int AssignValue(iomeConfig *CFG,char *item,char *err)
{
  return atoi(AssignString(CFG,item,err));
}

a4cCOLOR AssignColor(iomeConfig *CFG,char *item,char *err)
{
  a4cCOLOR c;
  char * v = AssignString(CFG,item,err);
  if (sscanf(v,"(%d,%d,%d)",&c.r,&c.g,&c.b)!=3) die(err);
  return c;
}

char *MODES[]={
  "1024x768x256",
  "800x600x256",
  "640x480x256",
  NULL
};

void LoadConfigure(int argc,char **argv,ttyDime *DIM)
{
  char * tmp;
  char buff[1024];
  int r,n;
  
  memset(DIM,0,sizeof(ttyDime));

  r=iomeDefaultLoader(&DIM->CONF);
  if (r!=1) die("Load Error %d",r);
  tmp = iomeWhichItem(&DIM->CONF.SYS,"tty");
  if (tmp==NULL) die("Can not find tty configure file!\n");
  r = iomeLoadConfiguration(tmp,&DIM->CFG);
  if (r==0) die("%s - Configure file error!",tmp);

  /* Charset */
  tmp = AssignString(&DIM->CONF.SYS,"lang","No charset specified!");
  if (strcmp(tmp,"big5")==0)   DIM->CC = isBig5; 
    else
  if (strcmp(tmp,"gb2312")==0) DIM->CC = isGB2312; 
    else die("Well, bogie!");
  
  DIM->base_charset = tmp;
  DIM->neutral_charset = "neutral";
  DIM->current_charset = DIM->neutral_charset;
  DIM->method_size = 1;
  DIM->filter_size = 1;
  DIM->fontsize_name[0] = iomeWhichItem(&DIM->CONF.CONF,"size.half");
  DIM->fontsize_name[1] = iomeWhichItem(&DIM->CONF.CONF,"size.full");
  DIM->METHOD_ENGLISH   = iomeWhichItem(&DIM->CONF.CONF,"method.english");
  DIM->FILTER_NEUTRAL   = iomeWhichItem(&DIM->CONF.CONF,"charset.neutral");
  DIM->Method.message   = DIM->METHOD_ENGLISH;
  DIM->Charset.message  = DIM->FILTER_NEUTRAL;
  DIM->FontSize.message = DIM->fontsize_name[0];
  DIM->KeyArea.message  = (char *)malloc(1024);
  DIM->ListArea.message = (char *)malloc(1024);
  memset(DIM->KeyArea.message,0,1024);
  memset(DIM->ListArea.message,0,1024);
  tmp = AssignString(&DIM->CFG,"title","No title!");
  strcpy(DIM->ListArea.message,tmp);


  /* Create Video Mode List */
  for (n=0,r=0;n<DIM->CFG.size;n++) 
    if (strcmp(DIM->CFG.items[n].key,"video.mode")==0) r++;
  DIM->modelists = (char**)malloc(sizeof(char*)*(r+1));
  for (n=0,r=0;n<DIM->CFG.size;n++)
    if (strcmp(DIM->CFG.items[n].key,"video.mode")==0) {
      DIM->modelists[r] = DIM->CFG.items[n].value;
      r++;
    }
  DIM->modelists[r] = NULL;

  memset(&FONT,0,sizeof(FONT));
  tmp = AssignString(&DIM->CFG,"english.fontsize","English Font Size?");
  if (sscanf(tmp,"(%d,%d)",&FONT.eng_w,&FONT.eng_h)!=2) die("English Font Size?");

  tmp = AssignString(&DIM->CFG,"font.path","Font Path?");
  n = strlen(tmp);
  if (tmp[n-1]!='/') {
   sprintf(buff,"%s/",tmp);
   FONT.dbcs_path = strdup(buff);
  } else FONT.dbcs_path = tmp;
  
  FONT.dbcs_fname = AssignString(&DIM->CFG,"font.chinese","Chinese Font?");
  tmp = AssignString(&DIM->CFG,"font.english","English Font?");
  sprintf(buff,"%s%s",FONT.dbcs_path,tmp);
  FONT.eng_fname = strdup(buff);
  
  DIM->fgcolor = AssignColor(&DIM->CFG,"panel.fgcolor","Foreground Color?");
  DIM->bgcolor = AssignColor(&DIM->CFG,"panel.bgcolor","Background Color?");
  DIM->egcolor = AssignColor(&DIM->CFG,"panel.egcolor","English Color?");

  DIM->dev_console = AssignString(&DIM->CFG,"console.device","No /dev/console ?");
  DIM->dev_mouse   = AssignString(&DIM->CFG,"mouse.device","No /dev/mouse ?");

  tmp = AssignString(&DIM->CFG,"mouse.enable","mouse ???");
  mouse_enable = (strcmp(tmp,"enable")==0);

  a4cLoadModules(DIM);
  
  argv++;
  SetupTTY(DIM,DIM->modelists,DIM->dev_console,DIM->dev_mouse,argv);

  set_swcolor(DIM->TERM->ttyColor[ 0],  0,  0,  0);
  set_swcolor(DIM->TERM->ttyColor[ 1],128,  0,  0);
  set_swcolor(DIM->TERM->ttyColor[ 2],  0,128,  0);
  set_swcolor(DIM->TERM->ttyColor[ 3],128,128,  0);
  set_swcolor(DIM->TERM->ttyColor[ 4],  0,  0,128);
  set_swcolor(DIM->TERM->ttyColor[ 5],128,  0,128);
  set_swcolor(DIM->TERM->ttyColor[ 6],  0,128,128);
  set_swcolor(DIM->TERM->ttyColor[ 7],192,192,192);
  set_swcolor(DIM->TERM->ttyColor[ 8],128,128,128);
  set_swcolor(DIM->TERM->ttyColor[ 9],255,  0,  0);
  set_swcolor(DIM->TERM->ttyColor[10],  0,255,  0);
  set_swcolor(DIM->TERM->ttyColor[11],255,255,  0);
  set_swcolor(DIM->TERM->ttyColor[12],  0,  0,255);
  set_swcolor(DIM->TERM->ttyColor[13],255,  0,255);
  set_swcolor(DIM->TERM->ttyColor[14],  0,255,255);
  set_swcolor(DIM->TERM->ttyColor[15],255,255,255);
  DIM->TERM->frgnd = DIM->TERM->ttyColor[7];
  DIM->TERM->bkgnd = DIM->TERM->ttyColor[0];
  
  swDrawTerminal(DIM->TERM);
  ttyDrawPanel(DIM);

}

void SwitchFontSize(ttyDime *DIM)
{
  DIM->fontsize ^= 1;
  DIM->FontSize.message = DIM->fontsize_name[DIM->fontsize];
  ttyDrawPanel(DIM);
}

void SwitchOutputFilter(ttyDime *DIM)
{
  if (DIM->current_filter==0) {
    DIM->Charset.message = DIM->FILTER_NEUTRAL;
  } else {
    DIM->FILTER_NOW = DIM->FILTER[DIM->current_filter];
    DIM->Charset.message = DIM->FILTER_NOW->RECORD.FilterName;
  }
  ttyDrawPanel(DIM);
}

void NextOutputFilter(ttyDime *DIM)
{
  DIM->current_filter++;
  DIM->current_filter %= DIM->filter_size;
  SwitchOutputFilter(DIM);
}

void SwitchInputMethod(ttyDime *DIM)
{
  if (DIM->current_method==0) {
    DIM->current_charset = DIM->neutral_charset;
    DIM->Method.message = DIM->METHOD_ENGLISH;
    DIM->KeyArea.message[0]  = 0;
    DIM->ListArea.message[0] = 0;
  } else {
    DIM->METHOD_NOW = DIM->METHOD[DIM->current_method];
    DIM->METHOD_NOW->PREPARE(&DIM->METHOD_NOW->RECORD);
    DIM->Method.message = DIM->METHOD_NOW->RECORD.MethodName;
    DIM->current_charset = DIM->METHOD_NOW->RECORD.Encoding;
    DIM->KeyArea.message[0]  = 0;
    DIM->ListArea.message[0] = 0;
  }
  ttyDrawPanel(DIM);
}

void NextInputMethod(ttyDime *DIM)
{
  DIM->current_method++;
  DIM->current_method %= DIM->method_size;
  SwitchInputMethod(DIM);
}

void SwitchInputBetweenEnglish(ttyDime *DIM)
{
  if (DIM->current_method==0) {
    if (DIM->last_method==0) {
      DIM->current_method++;
      DIM->current_method %= DIM->method_size;
    } else {
      DIM->current_method = DIM->last_method;
    }
  } else {
    DIM->last_method = DIM->current_method;
    DIM->current_method = 0;
  }
  SwitchInputMethod(DIM);
}

void SendOutRespond(ttyDime *DIM,int R,int ch)
{
 char * buf;
  switch (R) {
    case IME_RT_REJECT:
      if (DIM->fontsize!=0&&ch>=' '&&ch<0x7e) {
        /* SendFullSizeFont(DIM,ch); */
        break;
      }
      /* SendReject */
    break;
    case IME_RT_BLOCKOUT: break;
    case IME_RT_SEND:
      buf = DIM->METHOD_NOW->RECORD.Value;
      if (strlen(buf)>0) {
        UnifyCharset(DIM,buf);
        write(DIM->pty,buf,strlen(buf));
        buf[0] = 0;
      } else { /* SendZeroOut */ }
    break;
  }
  strcpy(DIM->KeyArea.message,DIM->METHOD_NOW->RECORD.KeyArea);
  strcpy(DIM->ListArea.message,DIM->METHOD_NOW->RECORD.ListArea);
  UnifyCharset(DIM,DIM->KeyArea.message);
  UnifyCharset(DIM,DIM->ListArea.message);
}

void GoForMethod(ttyDime * DIM,int ch,int state)
{
  int R;
  
  R = DIM->METHOD_NOW->FEEDKEY(&DIM->METHOD_NOW->RECORD,ch,state);
  SendOutRespond(DIM,R&IME_RT_MASK,ch);
  ttyDrawPanel(DIM);
  if ((R&IME_RT_CRY)==IME_RT_CRY) { /* Make Noise */ }
}

void CloseME(ttyDime *DIM)
{
  if (swNoticeYesNo(
        "All For Chinese TTY V0.1.0",
        "Yes","No",
        "You are about to leave a4c.tty\n\n"
        "             Quit ?"
      )
     )
    CloseTTY(DIM);
}

void CastingKey(ttyDime *DIM)
{
  int key;
  int state;
  int c;
  int shift,alt,ctrl;
  unsigned char ww;
  
  c = swGetKey();
  if (c<0) return;
  ww = (unsigned char)c;
  ttyConvertKey(c,&key,&state);
  shift = state & IME_KF_SHIFT;
  alt   = state & IME_KF_ALT;
  ctrl  = state & IME_KF_CONTROL;
  
//  swMessageBox("CastingKey","OK","ENTER: %c",c);
  
  if (ctrl&&alt&&key=='~') {
    swMessageBox("Grab Screen Image","OK","Print Screen");
  } else
  if (ctrl&&alt&&key==IME_KEY_BACK) {
    CloseME(DIM);
  } else
  if (shift&&alt) {
    /* Do something here */
  } else
  if (ctrl&&alt) {
    /* Do something here */
  } else
  if (shift&&ctrl&&key==0) { /* CTRL + SHIFT - Next Input Method */
    NextInputMethod(DIM);
  } else
  if (ctrl&&key==' ') { /* CTRL + SPACE - English And IME */
    SwitchInputBetweenEnglish(DIM);
  } else
  if (ctrl&&key=='~') { /* CTRL + '~' - Next OME */
    NextOutputFilter(DIM);
  } else
  if (shift&&key==' ') { /* SHIFT + SPACE - FontSize */
    SwitchFontSize(DIM);
  } else
  if (DIM->current_method>0) GoForMethod(DIM,key,state); 
    else 
  if (DIM->current_method==0) {
    if (DIM->fontsize==1) { /* Send Full Font */
    } else {
      write(DIM->pty,&ww,sizeof(char));
    }
  }
}

void main(int argc,char **argv)
{
  int c,id;
  char buf[1024];
  ttyDime DIM;
  
  int n=0;
  
  LoadConfigure(argc,argv,&DIM);
  ttySetupKeymap(swConsole);
  
  for (;;) {

    switch (ttyNextEvent(&DIM)) {
      case TTY_EVENT_MICE_MENU:
        mice_recover();
        ttyDrawMenuButton(&DIM,0);
        mice_move(mouse_x,mouse_y);
        
        id = swDoMenu(DIM.Menu.x1,DIM.Menu.y1,GRAVITY_SW,&MENU);
        switch (id) {
          case 105:
            swMessageBox(
              "A4C.TTY Module Loader","OK",
              "No module available now!\n\n"
              "Module is used when the system\n"
              "need an extra service such as\n"
              "Configurator."
            );
          break;
          case 997:
            swMessageBox(
              "Information about this section","OK",
              "Terminal: %dx%d\n"
              "Video Mode: %s\n"
              "Font Size: %dx%d\n"
              "English Font: %dx%d\n"
              "Input Method: %d\n"
              "Output Filter: %d\n"
              "ScanLineWidth: %d",
              DIM.row,DIM.col,DIM.VideoMode,
              FONT.width,FONT.height,
              FONT.eng_w,FONT.eng_h,
              DIM.method_size,DIM.filter_size,
              swScanWidth
            );
          break;
          case 998:
            swMessageBox(
              "About A4C.TTY","OK",
              "All For Chinese TTY\n"
              "Author: Brian Lin\n"
              "Email: foxman@okstation.com\n"
              "Version: 0.1.0"
            );
          break;
          case 999: CloseME(&DIM); break; 
          case 101:
//            vga_setdisplaystart(0);
//            write_cr(&DIM);
          break;
        }
        mice_recover();
        ttyDrawMenuButton(&DIM,1);
        mice_move(mouse_x,mouse_y);
      break;
      case TTY_EVENT_MICE_SCROLL:
        swMessageBox("SCROLL BAR","OK","Not implement yet!");
      break;
      case TTY_EVENT_MICE_METHOD:
        if (mice_button&MOUSE_LEFTBUTTON) NextInputMethod(&DIM); else
        if (mice_button&MOUSE_RIGHTBUTTON) { /* Menu Selection */ }
      break;
      case TTY_EVENT_MICE_CHARSET:
        if (mice_button&MOUSE_LEFTBUTTON) NextOutputFilter(&DIM); else
        if (mice_button&MOUSE_RIGHTBUTTON) { /* Menu Selection */ }
      break;
      case TTY_EVENT_MICE_FONTSIZE:
        if (mice_button&MOUSE_LEFTBUTTON) SwitchFontSize(&DIM); else
        if (mice_button&MOUSE_RIGHTBUTTON) { /* Menu Selection */ }        
      break;
      case TTY_EVENT_MICE_TERM:
        /* swTermMouseOver(...); */
      break;
      case TTY_EVENT_KEYPRESS:
        CastingKey(&DIM);
      break;
      case TTY_EVENT_VTMSG:
        c = read(DIM.pty,buf,1024);
        swTermAdd(DIM.TERM,buf,c);
      break;
    }
  }
}


