#include "iome.h"

int iomeTOUPPER(int ch)
{
  if (ch>=0x100) return ch;
  return toupper(ch);
}

int iomeTOLOWER(int ch)
{
  if (ch>=0x100) return ch;
  return tolower(ch);
}

char * cocktail(char *p)
{
  char * tail;
  if ((*p)==0) return NULL;
  /* find first key */
  while (*p&&*p!=' '&&*p!=9) p++;
  if (*p==0) return NULL;
  *p = 0; p++;
  /* find separator */
  while (*p&&(*p==' '||*p==9)) p++;
  if (*p==0) return NULL;
  tail = p;
  /* cut tail */
  while (*p!=0&&*p!=10&&*p!=13) p++;
  *p = 0;
  return tail;
}

int iomeLoadConfiguration(char *FileName,iomeConfig *CONF)
{
  FILE *fp;
  char buff[IOME_MAX_BUFF];
  char *p;
  int valid;

  fp = fopen(FileName,"r");
  if (fp==NULL) return CONF->size;
  do {
    valid = (fgets(buff,IOME_MAX_BUFF,fp)!=NULL);
    if (valid&&(p=cocktail(buff))!=NULL) {
      CONF->items = (iomeConfItem *)realloc(CONF->items,sizeof(iomeConfItem)*(CONF->size+1));
      CONF->items[CONF->size].key   = strdup(buff);
      CONF->items[CONF->size].value = strdup(p);
      CONF->size++;
    }
  } while (valid);
  fclose(fp);
  return CONF->size;
}

int iomeFreeConfiguration(iomeConfig *CONF)
{
  if (CONF==NULL) return 0;
  while (CONF->size>0) {
    CONF->size--;
    free(CONF->items[CONF->size].key);
    free(CONF->items[CONF->size].value);
  }
  free(CONF->items);
  return 1;
}

char * iomeWhichItem(iomeConfig *CONF,char *key)
{
  int n=0;
  while (n<CONF->size&&strcmp(CONF->items[n].key,key)!=0) n++;
  return (n<CONF->size) ? CONF->items[n].value : NULL;
}

int iomeDefaultLoader(iomeDefaultConf * DEFCONF)
{
  char *home;
  char DIR[256];
  char conf[IOME_MAX_BUFF];
  char * cfg;
  char * modules;

  home = getenv("HOME");
  if (home==NULL) home = getenv("home");
  if (home!=NULL) {
    sprintf(DIR,"%s/.iome",home);
  } else {
    sprintf(DIR,"/etc/iome");
  }
  sprintf(conf,"%s/configure",DIR);
  if (access(conf,R_OK)!=0) {
    sprintf(DIR,"/etc/iome");
    sprintf(conf,"%s/configure",DIR);
    if (access(conf,R_OK)!=0) return -1;
  }
  iomeLoadConfiguration(conf,&DEFCONF->SYS);
  cfg     = iomeWhichItem(&DEFCONF->SYS,"configure");
  modules = iomeWhichItem(&DEFCONF->SYS,"modules");
  if (cfg!=NULL) {
    sprintf(conf,"%s/%s",DIR,cfg);
    if (access(conf,R_OK)!=0) return -2;
    iomeLoadConfiguration(conf,&DEFCONF->CONF);
  } else return -2;
  if (modules!=NULL) {
    sprintf(conf,"%s/%s",DIR,modules);
    if (access(conf,R_OK)!=0) return -3;
    iomeLoadConfiguration(conf,&DEFCONF->MODULES);
  } else return -3;
  return 1;
}

#define isChineseChar(c) (((c)>=0x81)&&((c)<=0xfe))
int isBig5(unsigned char ch[])
{
  return (
           isChineseChar(ch[0])&&
           ((0x40<=ch[1])&&(ch[1]<=0xfe))
         ) ? 2:0;
}

int isGB2312(unsigned char ch[])
{
  return (isChineseChar(ch[0])&&(ch[1]>=0x80&&ch[1]<=0xfe)) ? 2:0;
}
